<?php declare(strict_types=1);

namespace Test\Unit\Query;

use BitBadger\PDODocument\Configuration;
use BitBadger\PDODocument\DocumentException;
use BitBadger\PDODocument\Field;
use BitBadger\PDODocument\Mode;
use BitBadger\PDODocument\Query\Patch;
use PHPUnit\Framework\Attributes\TestDox;
use PHPUnit\Framework\TestCase;

/**
 * Unit tests for the Patch class
 */
class PatchTest extends TestCase
{
    #[TestDox('By ID succeeds for PostgreSQL')]
    public function testByIdSucceedsForPostgreSQL(): void
    {
        try {
            Configuration::$mode = Mode::PgSQL;
            $this->assertEquals("UPDATE doc_table SET data = data || @data WHERE data->>'id' = @id",
                Patch::byId('doc_table'), 'Patch UPDATE statement is not correct');
        } finally {
            Configuration::$mode = null;
        }
    }

    #[TestDox('By ID succeeds for SQLite')]
    public function testByIdSucceedsForSQLite(): void
    {
        try {
            Configuration::$mode = Mode::SQLite;
            $this->assertEquals("UPDATE my_table SET data = json_patch(data, json(@data)) WHERE data->>'id' = @id",
                Patch::byId('my_table'), 'Patch UPDATE statement is not correct');
        } finally {
            Configuration::$mode = null;
        }
    }

    #[TestDox('By ID fails when mode not set')]
    public function testByIdFailsWhenModeNotSet(): void
    {
        $this->expectException(DocumentException::class);
        Configuration::$mode = null;
        Patch::byId('oof');
    }

    #[TestDox('By fields succeeds for PostgreSQL')]
    public function testByFieldsSucceedsForPostgreSQL(): void
    {
        try {
            Configuration::$mode = Mode::PgSQL;
            $this->assertEquals("UPDATE that SET data = data || @data WHERE data->>'something' < @some",
                Patch::byFields('that', [Field::LT('something', 17, '@some')]),
                'Patch UPDATE statement is not correct');
        } finally {
            Configuration::$mode = null;
        }
    }

    #[TestDox('By fields succeeds for SQLite')]
    public function testByFieldsSucceedsForSQLite(): void
    {
        try {
            Configuration::$mode = Mode::SQLite;
            $this->assertEquals(
                "UPDATE a_table SET data = json_patch(data, json(@data)) WHERE data->>'something' > @it",
                Patch::byFields('a_table', [Field::GT('something', 17, '@it')]),
                'Patch UPDATE statement is not correct');
        } finally {
            Configuration::$mode = null;
        }
    }

    public function testByFieldsFailsWhenModeNotSet(): void
    {
        $this->expectException(DocumentException::class);
        Configuration::$mode = null;
        Patch::byFields('oops', []);
    }
}
